//
// GameX - SoundX Class Header 
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source License. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

//#include "gamex-win-dx.h"


#ifndef GAMEX_SOUND_H_INCLUDED
#define GAMEX_SOUND_H_INCLUDED

#include "gamex-global-def.h"


class SoundX
{
public:
	SoundX ();
	~SoundX ();
	bool LoadWav (const char *filename);
	inline bool Load (const char *filename) {return LoadWav(filename);}
	void Unload ();
	void InvalidWav ();
	bool ReadName (FILE* wav, const char *match_name);
	bool ReadSize (FILE* wav, unsigned int &size);
///		void CutLength (float new_length);
	bool NeedUpdate () const;
	inline unsigned short	GetBPS () const			{return z_bits_per_sample;}
	inline unsigned short	GetNumChannels () const	{return z_num_channels;}
	inline int				GetNumSamples () const	{return z_num_samples;}
	inline int				GetSamplesPerSec () const	{return z_samples_per_sec;}
	inline int				GetNumBytes () const		{return z_num_bytes;}
//		inline int				GetIndex () const			{return dx_index;}
//		inline void				SetIndex (int n) const		{dx_index = n;}
	inline char*			GetData () const			{return z_data;}
	inline bool				GetRefresh () const		{return z_dx_refresh;}
	inline void				SetRefresh (bool r) const		{z_dx_refresh = r;}
	inline LPDIRECTSOUNDBUFFER	GetDSBuffer () const	{return z_ds_sound_buffer;}
	void PrepareDSBuffer ()  const;
	inline const char*	GetFilename () const	{return z_filename;}
private:
	unsigned int	z_samples_per_sec;	// Quality (44100, 22050, 11025, 8000, 6000, etc.) 
	unsigned short	z_bits_per_sample;	// Resolution (8-bit, 16-bit, etc.)
	unsigned short	z_num_channels;		// Number of channels
	float			z_length;				// Length of sound (seconds)
	int				z_num_samples;		// Num Samples = z_samples_per_sec * length
	int				z_num_bytes;			// Total Bytes = (z_bits_per_sample/8) * z_num_channels * z_num_samples
	mutable char*			z_data;
	mutable LPDIRECTSOUNDBUFFER z_ds_sound_buffer;
//		int				dx_index;			// Direct Sound index
	mutable bool			z_dx_refresh;			// Direct Sound refresh
	char			z_filename [MAX_PATH];
};
typedef	SoundX*		SoundXPtr;

static int now_playing;
static int merge_id;

class MusicX
{
public:
	MusicX ();
	virtual ~MusicX ();

	// loads a music file
	// The game's code should call this function to load a music object before playing it,
	// i.e. call music.Load("musicfile.mid"); before calling GameX.Play(&music);
	virtual bool Load (char *filename);

	// Getter methods:
	inline IDirectMusicSegment * GetSegment() const {return z_segment;}
	inline IDirectMusicSegmentState * GetSegmentState() {return z_segment_state;}
	inline float GetVolume() {return z_vol;} 
	inline int GetPlayState() {return z_playstate;} 
	void SetPlayState(int set) {z_playstate = set;} // to be called by GameX only
	void RememberVol(float set) {z_vol = set;} // to be called by GameX only

	// called by GameX when uninitializing DirectX
	// necessary because the deconstructor can be called AFTER DirectX is uninitialized
	// but music objects must be unloaded BEFORE it's uninitialized or there's a runtime error
	void Release ();

	// Runs the music through a volume/pitch/? curve
	// Used by GameX.VolumeFadeMusic() and GameX.PitchBendMusic(), not really intended to be called directly
	void Curve(long time, int target, int type, int shape, int start = -1);

	virtual void AdjustSize() {}

	int format; // MEDIA_TYPE_DMUSIC or MEDIA_TYPE_DSHOW
	int playing_loops_left; // for MP3 looping
	IGraphBuilder *	dsh_gb;
	IMediaControl * dsh_mc;
	IMediaPosition * dsh_mp;
	IBasicAudio * dsh_ba;
	IBasicVideo * dsh_bv;
	IVideoWindow * dsh_vw;
	IMediaEventEx * dsh_me;
	IDirectDrawVideo * dd_video;
protected:
	bool z_LoadWithDirectShow (char *filename);
	bool z_LoadWithDirectMusic (char *filename);
	virtual bool z_SetUpFilterGraph ();
	virtual bool z_PrepareLoaded ();
	virtual operator const char* () const {return "MusicX";} // convert-to-string operator

	IDirectMusicSegment *		z_segment;		// Music segment data
	IDirectMusicSegmentState *	z_segment_state;	// Music segment state
	bool						z_registered;
	float						z_vol;
	int							z_playstate;
};

class MovieX : public MusicX
{
public:
	// loads a movie file
	bool Load (char *filename);

	inline long GetOriginalWidth () { return z_xres; }
	inline long GetOriginalHeight () { return z_yres; }

	// set where the movie renders
	void SetPosition (int xLeft, int yTop, int xRight, int yBottom);
	void SetPosition (int xCenter, int yCenter, float scaleFactor = 2.0f); // another way to set the same thing

	void AdjustSize();

protected:
	long z_xres, z_yres;
	long z_x1, z_y1, z_x2, z_y2;
	bool z_SetUpFilterGraph();
	bool z_PrepareLoaded();
	operator const char* () const {return "MovieX";} // convert-to-string operator


public:
	friend class GameXClass;
};

#endif